function noop() {}

function assign(tar, src) {
	for (var k in src) tar[k] = src[k];
	return tar;
}

function assignTrue(tar, src) {
	for (var k in src) tar[k] = 1;
	return tar;
}

function append(target, node) {
	target.appendChild(node);
}

function insert(target, node, anchor) {
	target.insertBefore(node, anchor);
}

function detachNode(node) {
	node.parentNode.removeChild(node);
}

function destroyEach(iterations, detach) {
	for (var i = 0; i < iterations.length; i += 1) {
		if (iterations[i]) iterations[i].d(detach);
	}
}

function createElement(name) {
	return document.createElement(name);
}

function createText(data) {
	return document.createTextNode(data);
}

function createComment() {
	return document.createComment('');
}

function addListener(node, event, handler, options) {
	node.addEventListener(event, handler, options);
}

function removeListener(node, event, handler, options) {
	node.removeEventListener(event, handler, options);
}

function setAttribute(node, attribute, value) {
	if (value == null) node.removeAttribute(attribute);
	else node.setAttribute(attribute, value);
}

function setData(text, data) {
	text.data = '' + data;
}

function setStyle(node, key, value) {
	node.style.setProperty(key, value);
}

function addResizeListener(element, fn) {
	if (getComputedStyle(element).position === 'static') {
		element.style.position = 'relative';
	}

	const object = document.createElement('object');
	object.setAttribute('style', 'display: block; position: absolute; top: 0; left: 0; height: 100%; width: 100%; overflow: hidden; pointer-events: none; z-index: -1;');
	object.type = 'text/html';

	let win;

	object.onload = () => {
		win = object.contentDocument.defaultView;
		win.addEventListener('resize', fn);
	};

	if (/Trident/.test(navigator.userAgent)) {
		element.appendChild(object);
		object.data = 'about:blank';
	} else {
		object.data = 'about:blank';
		element.appendChild(object);
	}

	return {
		cancel: () => {
			win && win.removeEventListener && win.removeEventListener('resize', fn);
			element.removeChild(object);
		}
	};
}

function destroyBlock(block, lookup) {
	block.d(1);
	lookup[block.key] = null;
}

function updateKeyedEach(old_blocks, component, changed, get_key, dynamic, ctx, list, lookup, node, destroy, create_each_block, intro_method, next, get_context) {
	var o = old_blocks.length;
	var n = list.length;

	var i = o;
	var old_indexes = {};
	while (i--) old_indexes[old_blocks[i].key] = i;

	var new_blocks = [];
	var new_lookup = {};
	var deltas = {};

	var i = n;
	while (i--) {
		var child_ctx = get_context(ctx, list, i);
		var key = get_key(child_ctx);
		var block = lookup[key];

		if (!block) {
			block = create_each_block(component, key, child_ctx);
			block.c();
		} else if (dynamic) {
			block.p(changed, child_ctx);
		}

		new_blocks[i] = new_lookup[key] = block;

		if (key in old_indexes) deltas[key] = Math.abs(i - old_indexes[key]);
	}

	var will_move = {};
	var did_move = {};

	function insert(block) {
		block[intro_method](node, next);
		lookup[block.key] = block;
		next = block.first;
		n--;
	}

	while (o && n) {
		var new_block = new_blocks[n - 1];
		var old_block = old_blocks[o - 1];
		var new_key = new_block.key;
		var old_key = old_block.key;

		if (new_block === old_block) {
			// do nothing
			next = new_block.first;
			o--;
			n--;
		}

		else if (!new_lookup[old_key]) {
			// remove old block
			destroy(old_block, lookup);
			o--;
		}

		else if (!lookup[new_key] || will_move[new_key]) {
			insert(new_block);
		}

		else if (did_move[old_key]) {
			o--;

		} else if (deltas[new_key] > deltas[old_key]) {
			did_move[new_key] = true;
			insert(new_block);

		} else {
			will_move[old_key] = true;
			o--;
		}
	}

	while (o--) {
		var old_block = old_blocks[o];
		if (!new_lookup[old_block.key]) destroy(old_block, lookup);
	}

	while (n) insert(new_blocks[n - 1]);

	return new_blocks;
}

function blankObject() {
	return Object.create(null);
}

function destroy(detach) {
	this.destroy = noop;
	this.fire('destroy');
	this.set = noop;

	this._fragment.d(detach !== false);
	this._fragment = null;
	this._state = {};
}

function _differs(a, b) {
	return a != a ? b == b : a !== b || ((a && typeof a === 'object') || typeof a === 'function');
}

function fire(eventName, data) {
	var handlers =
		eventName in this._handlers && this._handlers[eventName].slice();
	if (!handlers) return;

	for (var i = 0; i < handlers.length; i += 1) {
		var handler = handlers[i];

		if (!handler.__calling) {
			try {
				handler.__calling = true;
				handler.call(this, data);
			} finally {
				handler.__calling = false;
			}
		}
	}
}

function flush(component) {
	component._lock = true;
	callAll(component._beforecreate);
	callAll(component._oncreate);
	callAll(component._aftercreate);
	component._lock = false;
}

function get() {
	return this._state;
}

function init(component, options) {
	component._handlers = blankObject();
	component._slots = blankObject();
	component._bind = options._bind;
	component._staged = {};

	component.options = options;
	component.root = options.root || component;
	component.store = options.store || component.root.store;

	if (!options.root) {
		component._beforecreate = [];
		component._oncreate = [];
		component._aftercreate = [];
	}
}

function on(eventName, handler) {
	var handlers = this._handlers[eventName] || (this._handlers[eventName] = []);
	handlers.push(handler);

	return {
		cancel: function() {
			var index = handlers.indexOf(handler);
			if (~index) handlers.splice(index, 1);
		}
	};
}

function set(newState) {
	this._set(assign({}, newState));
	if (this.root._lock) return;
	flush(this.root);
}

function _set(newState) {
	var oldState = this._state,
		changed = {},
		dirty = false;

	newState = assign(this._staged, newState);
	this._staged = {};

	for (var key in newState) {
		if (this._differs(newState[key], oldState[key])) changed[key] = dirty = true;
	}
	if (!dirty) return;

	this._state = assign(assign({}, oldState), newState);
	this._recompute(changed, this._state);
	if (this._bind) this._bind(changed, this._state);

	if (this._fragment) {
		this.fire("state", { changed: changed, current: this._state, previous: oldState });
		this._fragment.p(changed, this._state);
		this.fire("update", { changed: changed, current: this._state, previous: oldState });
	}
}

function _stage(newState) {
	assign(this._staged, newState);
}

function callAll(fns) {
	while (fns && fns.length) fns.shift()();
}

function _mount(target, anchor) {
	this._fragment[this._fragment.i ? 'i' : 'm'](target, anchor || null);
}

var proto = {
	destroy,
	get,
	fire,
	on,
	set,
	_recompute: noop,
	_set,
	_stage,
	_mount,
	_differs
};

const DeepDiff = require('deep-diff');
const applyChange = DeepDiff.applyChange;
const diff = DeepDiff.diff;
/**
 * Edit history tracker for a javascript object using deep-diff to generate and apply patches
 */
class EditHistory {
  /**
   * Instantiates an instance of EditHistory
   * @param {Object} obj The object or array to track
   */
  constructor(obj) {
    this.obj = JSON.parse(JSON.stringify(obj));
    
    // initialize arrays for forwards and backwards patches
    this.forward = [];
    this.backward = [];
  }

  /**
   * Clears all forward and backward patches
   */
  clear() {
    this.forward = [];
    this.backward = [];
  }

  /**
   * Records a change to an object
   * @param {Object} newObj The new object
   */
  recordChange(newObj) {
    const patch = {
      redo: diff(this.obj, newObj),
      undo: diff(newObj, this.obj)
    };

    this.obj = JSON.parse(JSON.stringify(newObj));

    if (!patch.redo || !patch.undo) {
      console.warn('Objects could not be diffed');
    }
    else {
      this.backward.push(patch);
    }
  }

  /**
   * Applies the most recent undo patch and returns the new object
   * @returns {Object} The tracked object
   */
  undo() {
    if (this.backward.length === 0) {
      return null;
    }

    // grab the most recent backwards patch
    const patch = this.backward.pop();

    // applyChange doesn't accept arrays, only its members
    patch.undo.forEach(x => applyChange(this.obj, x));

    // put the patch into the forward queue
    this.forward.push(patch);

    return JSON.parse(JSON.stringify(this.obj));
  }

  /**
   * Applies the most recent redo patch and returns the new object
   * @returns {Object} The tracked object
   */
  redo() {
    if (this.forward.length === 0) {
      return null;
    }

    // grab the most recent forwards patch
    const patch = this.forward.pop();

    // applyChange doesn't accept arrays, only its members
    patch.redo.forEach(x => applyChange(this.obj, x));

    // put the patch into the backward queue
    this.backward.push(patch);

    return JSON.parse(JSON.stringify(this.obj));
  }

  /**
   * Applies all the undo patches in the queue and returns the new object
   * @returns {Object} The tracked object
   */
  undoAll() {
    while (this.backward.length > 0) {
      this.undo();
    }

    return this.obj;
  }

  /**
   * Applies all the redo patches in the queue and returns the new object
   * @returns {Object} The tracked object
   */
  redoAll() {
    while (this.forward.length > 0) {
      this.redo();
    }
  }
}

var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var deepDiff = createCommonjsModule(function (module, exports) {
(function(root, factory) { // eslint-disable-line no-extra-semi
  var deepDiff = factory(root);
  // eslint-disable-next-line no-undef
  {
      // Node.js or ReactNative
      module.exports = deepDiff;
  }
}(commonjsGlobal, function(root) {
  var validKinds = ['N', 'E', 'A', 'D'];

  // nodejs compatible on server side and in the browser.
  function inherits(ctor, superCtor) {
    ctor.super_ = superCtor;
    ctor.prototype = Object.create(superCtor.prototype, {
      constructor: {
        value: ctor,
        enumerable: false,
        writable: true,
        configurable: true
      }
    });
  }

  function Diff(kind, path) {
    Object.defineProperty(this, 'kind', {
      value: kind,
      enumerable: true
    });
    if (path && path.length) {
      Object.defineProperty(this, 'path', {
        value: path,
        enumerable: true
      });
    }
  }

  function DiffEdit(path, origin, value) {
    DiffEdit.super_.call(this, 'E', path);
    Object.defineProperty(this, 'lhs', {
      value: origin,
      enumerable: true
    });
    Object.defineProperty(this, 'rhs', {
      value: value,
      enumerable: true
    });
  }
  inherits(DiffEdit, Diff);

  function DiffNew(path, value) {
    DiffNew.super_.call(this, 'N', path);
    Object.defineProperty(this, 'rhs', {
      value: value,
      enumerable: true
    });
  }
  inherits(DiffNew, Diff);

  function DiffDeleted(path, value) {
    DiffDeleted.super_.call(this, 'D', path);
    Object.defineProperty(this, 'lhs', {
      value: value,
      enumerable: true
    });
  }
  inherits(DiffDeleted, Diff);

  function DiffArray(path, index, item) {
    DiffArray.super_.call(this, 'A', path);
    Object.defineProperty(this, 'index', {
      value: index,
      enumerable: true
    });
    Object.defineProperty(this, 'item', {
      value: item,
      enumerable: true
    });
  }
  inherits(DiffArray, Diff);

  function arrayRemove(arr, from, to) {
    var rest = arr.slice((to || from) + 1 || arr.length);
    arr.length = from < 0 ? arr.length + from : from;
    arr.push.apply(arr, rest);
    return arr;
  }

  function realTypeOf(subject) {
    var type = typeof subject;
    if (type !== 'object') {
      return type;
    }

    if (subject === Math) {
      return 'math';
    } else if (subject === null) {
      return 'null';
    } else if (Array.isArray(subject)) {
      return 'array';
    } else if (Object.prototype.toString.call(subject) === '[object Date]') {
      return 'date';
    } else if (typeof subject.toString === 'function' && /^\/.*\//.test(subject.toString())) {
      return 'regexp';
    }
    return 'object';
  }

  // http://werxltd.com/wp/2010/05/13/javascript-implementation-of-javas-string-hashcode-method/
  function hashThisString(string) {
    var hash = 0;
    if (string.length === 0) { return hash; }
    for (var i = 0; i < string.length; i++) {
      var char = string.charCodeAt(i);
      hash = ((hash << 5) - hash) + char;
      hash = hash & hash; // Convert to 32bit integer
    }
    return hash;
  }

  // Gets a hash of the given object in an array order-independent fashion
  // also object key order independent (easier since they can be alphabetized)
  function getOrderIndependentHash(object) {
    var accum = 0;
    var type = realTypeOf(object);

    if (type === 'array') {
      object.forEach(function (item) {
        // Addition is commutative so this is order indep
        accum += getOrderIndependentHash(item);
      });

      var arrayString = '[type: array, hash: ' + accum + ']';
      return accum + hashThisString(arrayString);
    }

    if (type === 'object') {
      for (var key in object) {
        if (object.hasOwnProperty(key)) {
          var keyValueString = '[ type: object, key: ' + key + ', value hash: ' + getOrderIndependentHash(object[key]) + ']';
          accum += hashThisString(keyValueString);
        }
      }

      return accum;
    }

    // Non object, non array...should be good?
    var stringToHash = '[ type: ' + type + ' ; value: ' + object + ']';
    return accum + hashThisString(stringToHash);
  }

  function deepDiff(lhs, rhs, changes, prefilter, path, key, stack, orderIndependent) {
    changes = changes || [];
    path = path || [];
    stack = stack || [];
    var currentPath = path.slice(0);
    if (typeof key !== 'undefined' && key !== null) {
      if (prefilter) {
        if (typeof (prefilter) === 'function' && prefilter(currentPath, key)) {
          return;
        } else if (typeof (prefilter) === 'object') {
          if (prefilter.prefilter && prefilter.prefilter(currentPath, key)) {
            return;
          }
          if (prefilter.normalize) {
            var alt = prefilter.normalize(currentPath, key, lhs, rhs);
            if (alt) {
              lhs = alt[0];
              rhs = alt[1];
            }
          }
        }
      }
      currentPath.push(key);
    }

    // Use string comparison for regexes
    if (realTypeOf(lhs) === 'regexp' && realTypeOf(rhs) === 'regexp') {
      lhs = lhs.toString();
      rhs = rhs.toString();
    }

    var ltype = typeof lhs;
    var rtype = typeof rhs;
    var i, j, k, other;

    var ldefined = ltype !== 'undefined' ||
      (stack && (stack.length > 0) && stack[stack.length - 1].lhs &&
        Object.getOwnPropertyDescriptor(stack[stack.length - 1].lhs, key));
    var rdefined = rtype !== 'undefined' ||
      (stack && (stack.length > 0) && stack[stack.length - 1].rhs &&
        Object.getOwnPropertyDescriptor(stack[stack.length - 1].rhs, key));

    if (!ldefined && rdefined) {
      changes.push(new DiffNew(currentPath, rhs));
    } else if (!rdefined && ldefined) {
      changes.push(new DiffDeleted(currentPath, lhs));
    } else if (realTypeOf(lhs) !== realTypeOf(rhs)) {
      changes.push(new DiffEdit(currentPath, lhs, rhs));
    } else if (realTypeOf(lhs) === 'date' && (lhs - rhs) !== 0) {
      changes.push(new DiffEdit(currentPath, lhs, rhs));
    } else if (ltype === 'object' && lhs !== null && rhs !== null) {
      for (i = stack.length - 1; i > -1; --i) {
        if (stack[i].lhs === lhs) {
          other = true;
          break;
        }
      }
      if (!other) {
        stack.push({ lhs: lhs, rhs: rhs });
        if (Array.isArray(lhs)) {
          // If order doesn't matter, we need to sort our arrays
          if (orderIndependent) {
            lhs.sort(function (a, b) {
              return getOrderIndependentHash(a) - getOrderIndependentHash(b);
            });

            rhs.sort(function (a, b) {
              return getOrderIndependentHash(a) - getOrderIndependentHash(b);
            });
          }
          i = rhs.length - 1;
          j = lhs.length - 1;
          while (i > j) {
            changes.push(new DiffArray(currentPath, i, new DiffNew(undefined, rhs[i--])));
          }
          while (j > i) {
            changes.push(new DiffArray(currentPath, j, new DiffDeleted(undefined, lhs[j--])));
          }
          for (; i >= 0; --i) {
            deepDiff(lhs[i], rhs[i], changes, prefilter, currentPath, i, stack, orderIndependent);
          }
        } else {
          var akeys = Object.keys(lhs);
          var pkeys = Object.keys(rhs);
          for (i = 0; i < akeys.length; ++i) {
            k = akeys[i];
            other = pkeys.indexOf(k);
            if (other >= 0) {
              deepDiff(lhs[k], rhs[k], changes, prefilter, currentPath, k, stack, orderIndependent);
              pkeys[other] = null;
            } else {
              deepDiff(lhs[k], undefined, changes, prefilter, currentPath, k, stack, orderIndependent);
            }
          }
          for (i = 0; i < pkeys.length; ++i) {
            k = pkeys[i];
            if (k) {
              deepDiff(undefined, rhs[k], changes, prefilter, currentPath, k, stack, orderIndependent);
            }
          }
        }
        stack.length = stack.length - 1;
      } else if (lhs !== rhs) {
        // lhs is contains a cycle at this element and it differs from rhs
        changes.push(new DiffEdit(currentPath, lhs, rhs));
      }
    } else if (lhs !== rhs) {
      if (!(ltype === 'number' && isNaN(lhs) && isNaN(rhs))) {
        changes.push(new DiffEdit(currentPath, lhs, rhs));
      }
    }
  }

  function observableDiff(lhs, rhs, observer, prefilter, orderIndependent) {
    var changes = [];
    deepDiff(lhs, rhs, changes, prefilter, null, null, null, orderIndependent);
    if (observer) {
      for (var i = 0; i < changes.length; ++i) {
        observer(changes[i]);
      }
    }
    return changes;
  }

  function orderIndependentDeepDiff(lhs, rhs, changes, prefilter, path, key, stack) {
    return deepDiff(lhs, rhs, changes, prefilter, path, key, stack, true);
  }

  function accumulateDiff(lhs, rhs, prefilter, accum) {
    var observer = (accum) ?
      function (difference) {
        if (difference) {
          accum.push(difference);
        }
      } : undefined;
    var changes = observableDiff(lhs, rhs, observer, prefilter);
    return (accum) ? accum : (changes.length) ? changes : undefined;
  }

  function accumulateOrderIndependentDiff(lhs, rhs, prefilter, accum) {
    var observer = (accum) ?
      function (difference) {
        if (difference) {
          accum.push(difference);
        }
      } : undefined;
    var changes = observableDiff(lhs, rhs, observer, prefilter, true);
    return (accum) ? accum : (changes.length) ? changes : undefined;
  }

  function applyArrayChange(arr, index, change) {
    if (change.path && change.path.length) {
      var it = arr[index],
        i, u = change.path.length - 1;
      for (i = 0; i < u; i++) {
        it = it[change.path[i]];
      }
      switch (change.kind) {
        case 'A':
          applyArrayChange(it[change.path[i]], change.index, change.item);
          break;
        case 'D':
          delete it[change.path[i]];
          break;
        case 'E':
        case 'N':
          it[change.path[i]] = change.rhs;
          break;
      }
    } else {
      switch (change.kind) {
        case 'A':
          applyArrayChange(arr[index], change.index, change.item);
          break;
        case 'D':
          arr = arrayRemove(arr, index);
          break;
        case 'E':
        case 'N':
          arr[index] = change.rhs;
          break;
      }
    }
    return arr;
  }

  function applyChange(target, source, change) {
    if (typeof change === 'undefined' && source && ~validKinds.indexOf(source.kind)) {
      change = source;
    }
    if (target && change && change.kind) {
      var it = target,
        i = -1,
        last = change.path ? change.path.length - 1 : 0;
      while (++i < last) {
        if (typeof it[change.path[i]] === 'undefined') {
          it[change.path[i]] = (typeof change.path[i + 1] !== 'undefined' && typeof change.path[i + 1] === 'number') ? [] : {};
        }
        it = it[change.path[i]];
      }
      switch (change.kind) {
        case 'A':
          if (change.path && typeof it[change.path[i]] === 'undefined') {
            it[change.path[i]] = [];
          }
          applyArrayChange(change.path ? it[change.path[i]] : it, change.index, change.item);
          break;
        case 'D':
          delete it[change.path[i]];
          break;
        case 'E':
        case 'N':
          it[change.path[i]] = change.rhs;
          break;
      }
    }
  }

  function revertArrayChange(arr, index, change) {
    if (change.path && change.path.length) {
      // the structure of the object at the index has changed...
      var it = arr[index],
        i, u = change.path.length - 1;
      for (i = 0; i < u; i++) {
        it = it[change.path[i]];
      }
      switch (change.kind) {
        case 'A':
          revertArrayChange(it[change.path[i]], change.index, change.item);
          break;
        case 'D':
          it[change.path[i]] = change.lhs;
          break;
        case 'E':
          it[change.path[i]] = change.lhs;
          break;
        case 'N':
          delete it[change.path[i]];
          break;
      }
    } else {
      // the array item is different...
      switch (change.kind) {
        case 'A':
          revertArrayChange(arr[index], change.index, change.item);
          break;
        case 'D':
          arr[index] = change.lhs;
          break;
        case 'E':
          arr[index] = change.lhs;
          break;
        case 'N':
          arr = arrayRemove(arr, index);
          break;
      }
    }
    return arr;
  }

  function revertChange(target, source, change) {
    if (target && source && change && change.kind) {
      var it = target,
        i, u;
      u = change.path.length - 1;
      for (i = 0; i < u; i++) {
        if (typeof it[change.path[i]] === 'undefined') {
          it[change.path[i]] = {};
        }
        it = it[change.path[i]];
      }
      switch (change.kind) {
        case 'A':
          // Array was modified...
          // it will be an array...
          revertArrayChange(it[change.path[i]], change.index, change.item);
          break;
        case 'D':
          // Item was deleted...
          it[change.path[i]] = change.lhs;
          break;
        case 'E':
          // Item was edited...
          it[change.path[i]] = change.lhs;
          break;
        case 'N':
          // Item is new...
          delete it[change.path[i]];
          break;
      }
    }
  }

  function applyDiff(target, source, filter) {
    if (target && source) {
      var onChange = function (change) {
        if (!filter || filter(target, source, change)) {
          applyChange(target, source, change);
        }
      };
      observableDiff(target, source, onChange);
    }
  }

  Object.defineProperties(accumulateDiff, {

    diff: {
      value: accumulateDiff,
      enumerable: true
    },
    orderIndependentDiff: {
      value: accumulateOrderIndependentDiff,
      enumerable: true
    },
    observableDiff: {
      value: observableDiff,
      enumerable: true
    },
    orderIndependentObservableDiff: {
      value: orderIndependentDeepDiff,
      enumerable: true
    },
    orderIndepHash: {
      value: getOrderIndependentHash,
      enumerable: true
    },
    applyDiff: {
      value: applyDiff,
      enumerable: true
    },
    applyChange: {
      value: applyChange,
      enumerable: true
    },
    revertChange: {
      value: revertChange,
      enumerable: true
    },
    isConflict: {
      value: function () {
        return typeof $conflict !== 'undefined';
      },
      enumerable: true
    }
  });

  // hackish...
  accumulateDiff.DeepDiff = accumulateDiff;
  // ...but works with:
  // import DeepDiff from 'deep-diff'
  // import { DeepDiff } from 'deep-diff'
  // const DeepDiff = require('deep-diff');
  // const { DeepDiff } = require('deep-diff');

  if (root) {
    root.DeepDiff = accumulateDiff;
  }

  return accumulateDiff;
}));
});

/* src\data-grid.html generated by Svelte v2.16.1 */



const MIN_COLUMN_SIZE = 30;

/**
 * Computes the 'left' value for a grid-cell.
 * @param {Number} i The cell index
 * @param {Array} columnWidths The array of column widths in order
 * @returns {Number}
 */
function getCellLeft({i, columnWidths, __affixedColumnIndices, __scrollLeft}) {
  if (__affixedColumnIndices.indexOf(i) >= 0) {
    if (i === 0) {
      return __scrollLeft;
    }
    let left = __scrollLeft;
    for (let j = i-1; j >= 0; j--) {
      left += columnWidths[j];
    }
    return left;
  }

  let left = 0;
  for (let j = 0; j < i; j++) {
    left += columnWidths[j];
  }
  return left;
}

/**
 * Gets the closest column index given an x offset
 * @param {Number} x The x offset
 * @param {Array} columnWidths Array of column widths
 * @param {Array} __affixedColumnIndices Array of column indices that have been affixed
 * @param {Number} __scrollLeft The scrollLeft value of the scrollable container
 * @returns {Number}
 */ 
function getClosestIndex(x, columnWidths, __affixedColumnIndices, __scrollLeft) {
  let closest = 0;
  
  for (let i = 0; i < columnWidths.length; i++) {
    const left = getCellLeft({i, columnWidths, __affixedColumnIndices, __scrollLeft}) + Math.floor(columnWidths[i] / 2);
    
    if (left < x) {
      closest = i+1;
    }
  }

  // special handling required when there are affixed columns, because
  // if the grid is scrolled horizontally to the right, we want to have this function return the closest
  // affixed column, rather than any columns that might be closer to x but are being overlapped by an affixed column
  if (__affixedColumnIndices.length > 0) {
    const firstAffixedLeft = getCellLeft({i: __affixedColumnIndices[0], columnWidths, __affixedColumnIndices, __scrollLeft});
    const lastAffixedLeft = getCellLeft({i: __affixedColumnIndices[__affixedColumnIndices.length-1], columnWidths, __affixedColumnIndices, __scrollLeft});
    const lastAffixedRight = lastAffixedLeft + columnWidths[__affixedColumnIndices[__affixedColumnIndices.length-1]];
    const closestLeft = getCellLeft({i: closest, columnWidths, __affixedColumnIndices, __scrollLeft});
    if (closestLeft > firstAffixedLeft && closestLeft < lastAffixedRight) {
      if (closestLeft < lastAffixedRight && closestLeft > lastAffixedLeft) {
        closest = __affixedColumnIndices[__affixedColumnIndices.length-1];
      } else {
        for (let i = 0; i < __affixedColumnIndices.length; i++) {
          const left = getCellLeft({i: __affixedColumnIndices[i], columnWidths, __affixedColumnIndices, __scrollLeft}) + Math.floor(columnWidths[__affixedColumnIndices[i]]/2);

          if (left < x) {
            closest = __affixedColumnIndices[i] + 1;
          }
        }
      }
    }
  }

  return closest;
}

function getBodyScrollTop() {
 return window.pageYOffset || (document.documentElement.clientHeight ? document.documentElement.scrollTop : document.body.scrollTop);
}

function columnAffixLineLeft({ __scrollLeft, __affixedColumnIndices, columnWidths }) {
  // if no columns are affixed, set the line all the way to the left
  if (__affixedColumnIndices.length === 0) {
    return 0;
  }

  let left = __scrollLeft;
  for (let i = 0; i < __affixedColumnIndices.length; i++) {
    left += columnWidths[__affixedColumnIndices[i]];
  }

  return left;
}

function columnWidths_1({ columns }) {
  // if width was not provided for this column, give it a default value
  return columns.map(x => x.width || MIN_COLUMN_SIZE);
}

function numRows({ rows }) {
  return rows.length;
}

function gridSpaceWidth({ columnWidths, __resizing, __scrolledAllTheWayToTheRight }) {
  let sum = 0;
  for (let i = 0; i < columnWidths.length; i++) {
    sum += columnWidths[i];
  }

  /**
   * If the table is scrolled all the way to the right, resizing columns could
   * accelerate until the column is the minimum width. Add some extra space on the right
   * to ensure this undesired behavior does not happen. This doesn't seem like a perfect solution
   * but it works for now until I can think of something better.
   */ 
  if (__resizing && __scrolledAllTheWayToTheRight) {
    sum *= 2;
  }

  return sum;
}

function gridSpaceHeight({ rowHeight, numRows }) {
  return rowHeight * numRows;
}

function numRowsInViewport({ __innerOffsetHeight, rowHeight }) {
  return Math.ceil(__innerOffsetHeight / rowHeight);
}

function visibleRows({ rowHeight, rows, __scrollTop, numRowsInViewport, __extraRows }) {
  const start = Math.max(0, Math.floor((__scrollTop / rowHeight) - (__extraRows / 2)));
  const end = start + numRowsInViewport + __extraRows;

  return rows.slice(start, end).map((x, i) => {
    return {
      i: i + start, // for aria-rowindex
      data: x       // the row data
    };
  });
}

function data() {
  return {
    rows: [],                               // Rows to display
    columns: [],                            // Array of column definitions: { display: '', dataName: ''}, where display is what the display value is and dataName is what the key on the row object is
    rowHeight: 24,                          // Row height in pixels
    allowResizeFromTableCells: false,       // Allow the user to click on table cell borders to resize columns
    allowResizeFromTableHeaders: true,      // Allow the user to clikc on table header borders to resize columns
    allowColumnReordering: true,            // Allow the user to drag column headers to reorder columns
    allowColumnAffix: true,                 // Alow the user to affix columns to the left of the grid

    __extraRows: 0,                         // Number of extra rows to render beyond what is visible in the scrollable area
    __columnHeaderResizeCaptureWidth: 20,   // The width of the area on column borders that can be clicked to resize the column
    /**** Do not modify any of the data variables below ****/
    __affixedRowIndices: [],                // DO NOT MODIFY DIRECTLY. The row indices to affix to the top of the grid
    __affixedColumnIndices: [],             // DO NOT MODIFY DIRECTLY. The column indices to affix to the left side of the grid
    __affixingRow: false,                   // DO NOT MODIFY DIRECTLY. Whether a row affix operation is in progress
    __affixingColumn: false,                // DO NOT MODIFY DIRECTLY. Whether a column affix operation is in progress
    __rowActionLineTop: 0,                  // DO NOT MODIFY DIRECTLY. The 'top' position of the row action line
    __rowAffixLineTop: 0,                   // DO NOT MODIFY DIRECTLY. The 'top' position of the row affix line
    __columnAffixLineLeft: 0,               // DO NOT MODIFY DIRECTLY. The 'left' position of the column affix line
    __columnDragging: false,                // DO NOT MODIFY DIRECTLY. Whether a column is being dragged
    __columnIndexBeingDragged: null,        // DO NOT MODIFY DIRECTLY. The column index that is being dragged
    __columnDragOffsetX: 0,                 // DO NOT MODIFY DIRECTLY. The X offset of where the user clicked on the column header
    __resizing: false,                      // DO NOT MODIFY DIRECTLY. Whether or not a column is currently being resized
    __columnIndexBeingResized: null,        // DO NOT MODIFY DIRECTLY. The column index being resized
    __columnActionLineLeft: 0,              // DO NOT MODIFY DIRECTLY. The 'left' position of the action line
    __innerOffsetHeight: 0,                 // DO NOT MODIFY DIRECTLY. The height of the scrollable area on screen
    __scrollTop: 0,                         // DO NOT MODIFY DIRECTLY. The scrollTop position of the scrollable area
    __scrollLeft: 0,                        // DO NOT MODIFY DIRECTLY. The scrollLeft position of the scrollable area
    __scrolledAllTheWayToTheRight: false    // DO NOT MODIFY DIRECTLY. Whether the container is scrolled all the way to the right as of the last onscroll event
  };
}
function getCellZIndex(__affixedColumnIndices, i) {
  return __affixedColumnIndices.indexOf(i) === -1 ? 1 : 2;
}
function getRowTop(i, rowHeight) {
  return i * rowHeight;
}
var methods = {
  onWindowKeyDown(event) {
    if (event.ctrlKey) {

      if (event.keyCode === 90) {
        this.undo();
        event.preventDefault();
      }

      if (event.keyCode === 89) {
        this.redo();
        event.preventDefault();
      }
    }
  },

  /**
   * Event handler for window's mousemove event
   * @param {MouseEvent} event The MouseEvent object
   */
  onMouseMove(event) {
    this.onColumnDragMouseMove(event);
    this.onColumnResizeMouseMove(event);
    this.onRowAffixMouseMove(event);
    this.onColumnAffixMouseMove(event);
  },

  /**
   * Event handler for window's mouseup event
   * @param {MouseEvent} event The MouseEvent object
   */
  onMouseUp(event) {
    this.onColumnDragEnd(event);
    this.onColumnResizeEnd(event);
    this.onRowAffixEnd(event);
    this.onColumnAffixEnd(event);
  },

  /**
   * Event handler for when a value has been updated
   * @param {Object} event Event object with row and column objects
   */
  onCellUpdated(event) {
    const { rows } = this.get();
    rows[event.rowNumber][event.column.dataName] = event.value;

    this.set({ rows });
    this.fire('valueUpdated', event);
  },

  /**
   * Applies the most recent backward change
   */
  undo() {
    const rows = this.editHistory.undo();
    if (rows) {
      this.set({ rows, skipRecord: true });
    }
  },
  
  /**
   * Applies the most recent forward change
   */ 
  redo() {
    const rows = this.editHistory.redo();
    if (rows) {
      this.set({ rows, skipRecord: true });
    }
  },
  /**
   * Event handler for starting column affix operation
   */
  onColumnAffixStart(event) {
    // left click only
    if (event.which !== 1) {
      return;
    }
    const { __affixedColumnIndices, columnWidths } = this.get();
    if (__affixedColumnIndices.length > 0) {
      this.refs.tableSpace.scrollLeft = 0;
      this.set({
        __affixingColumn: true
      });
    } else {
      this.set({
        __affixingColumn: true
      });
    }    
  },

  /**
   * Event handler for mousemove column affix operation
   */
  onColumnAffixMouseMove(event) {
    const { __affixingColumn, __affixedColumnIndices, __scrollLeft, columnWidths } = this.get();
    if (!__affixingColumn) {
      return;
    }

    if (event.which !== 1) {
      this.onColumnAffixEnd(event);
      return;
    }

    const { left: wrapperPageX } = this.refs.wrapper.getBoundingClientRect();

    const offsetPoint = event.pageX - wrapperPageX + __scrollLeft;
    
    const idx = getClosestIndex(offsetPoint, columnWidths, __affixedColumnIndices, __scrollLeft);
    const indices = [];
    for (let i = 0; i < idx; i++) {
      indices.push(i);
    }

    this.set({
      __columnActionLineLeft: offsetPoint,
      __affixedColumnIndices: indices
    });
    
    event.preventDefault();
    
    // check to see if horizontal scroll position doesn't match where the 
  },

  /**
   * Event handler for ending column affix operation
   */
  onColumnAffixEnd(event) {
    this.set({
      __affixingColumn: false
    });
  },

  /**
   * Event handler for starting row affix operation
   */
  onRowAffixStart(event) {
    this.set({
      __affixingRow: true
    });
  },

  /**
   * Event handler for mousemove row affix operation
   */
  onRowAffixMouseMove(event) {
    const { __affixingRow } = this.get();
    if (!__affixingRow) {
      return;
    }
  },

  /**
   * Event handler for ending row affix operation
   */
  onRowAffixEnd(event) {
    this.set({
      __affixingRow: false
    });
  },

  /**
   * Event handler for column dragging
   */
  onColumnDragStart(event, columnIndex) {
    if (event.which !== 1) {
      return;
    }

    const { columnWidths, __scrollLeft, allowColumnReordering, __affixedColumnIndices } = this.get();
    
    // if the developer has disabled column reordering, don't begin a reorder
    if (!allowColumnReordering) {
      return;
    }

    this.set({
      __columnDragging: true,
      __columnIndexBeingDragged: columnIndex,
      __columnDragOffsetX: event.offsetX,
      __columnActionLineLeft: getCellLeft({i: columnIndex, columnWidths, __scrollLeft, __affixedColumnIndices}) - __scrollLeft
    });
  },
  onColumnDragMouseMove(event) {
    const { __columnDragging, __columnDragOffsetX, __scrollLeft, __affixedColumnIndices, __columnActionLineLeft, columnWidths } = this.get();
    if (!__columnDragging) {
      return;
    }

    // if user is no longer pressing the left mouse button and we are out of sync
    // with __columnDragging because mouseup didn't fire, finish the reorder
    if (event.which !== 1) {
      this.onColumnDragEnd(event);
      return;
    }


    const { left: wrapperPageX } = this.refs.wrapper.getBoundingClientRect();

    // change the position of the action line to the closest column index under the mouse
    const offsetPoint = event.pageX - wrapperPageX + __scrollLeft - __columnDragOffsetX;
    const idx = getClosestIndex(offsetPoint, columnWidths, __affixedColumnIndices, __scrollLeft);
    
    this.set({
      __columnActionLineLeft: getCellLeft({i: idx, columnWidths, __affixedColumnIndices, __scrollLeft}) - __scrollLeft
    });
  },

  /**
   * Window mouseup handler for column dragging
   */
  onColumnDragEnd(event) {
    const { __columnIndexBeingDragged, __scrollLeft, columnWidths, columns, __affixedColumnIndices, __columnDragging, __columnDragOffsetX } = this.get();

    // user might try to be clever and middle-click to scroll horizontally while dragging a column
    // don't stop the drag for middle clicks
    if (event.which !== 1) {
      return;
    }

    // if a column isn't being dragged, don't reorder anything
    if (!__columnDragging) {
      return;
    }

    const { left: wrapperPageX } = this.refs.wrapper.getBoundingClientRect();
    const offsetPoint = event.pageX - wrapperPageX + __scrollLeft - __columnDragOffsetX;

    // move column object to its new position in the array based off the mouse position and scroll position
    const newIdx = getClosestIndex(offsetPoint, columnWidths, __affixedColumnIndices, __scrollLeft);
    columns.splice(newIdx > __columnIndexBeingDragged ? newIdx - 1 : newIdx, 0, columns.splice(__columnIndexBeingDragged, 1)[0]);
    
    // delay firing of event so that new column order is accessible when handlers are called
    setTimeout(() => this.fire('columnOrderUpdated'), 0);

    this.set({
      __columnDragging: false,
      columns,
      __columnDragOffsetX: 0,
      __columnIndexBeingDragged: null
    });
  },

  /**
   * Mousedown handler for column resizing
   */
  onColumnResizeStart(event, columnIndex) {
    // left click only
    if (event.which !== 1) {
      return;
    }
    const { left: wrapperPageX } = this.refs.wrapper.getBoundingClientRect();
    const { __scrollLeft } = this.get();

    this.set({
      __resizing: true,
      __columnActionLineLeft: event.pageX - wrapperPageX - __scrollLeft,
      __columnIndexBeingResized: columnIndex
    });

    event.stopPropagation();
  },

  /**
   * Mousemove handler for column resizing
   */
  onColumnResizeMouseMove(event) {
    const { __resizing, __columnIndexBeingResized, columnWidths, __scrollLeft, __columnActionLineLeft, columns, __affixedColumnIndices } = this.get();

    // if not currently resizing a column, ignore the event
    if (!__resizing) {
      return;
    }

    const { left: wrapperPageX } = this.refs.wrapper.getBoundingClientRect();

    const resizeLineLeft = event.pageX - wrapperPageX;
    const columnLeft = getCellLeft({i: __columnIndexBeingResized, columnWidths, __affixedColumnIndices, __scrollLeft});
    const resizeLineMinLeft = columnLeft - __scrollLeft + MIN_COLUMN_SIZE;
    const newColumnWidth = Math.max((resizeLineLeft + __scrollLeft) - columnLeft, MIN_COLUMN_SIZE);
    // thanks to the virtual list, we're able to get away with setting the column's size while the mouse moves
    columns[__columnIndexBeingResized].width = newColumnWidth;

    const obj = {
      __columnActionLineLeft: Math.max(resizeLineLeft, resizeLineMinLeft),
      columns
    };
    
    // If mouseup was not fired for some reason, abort the resize
    if (event.which !== 1) {        
      obj.__resizing = false;
      obj.__columnIndexBeingResized = null;

      // delay firing the event until the next frame to guarantee that new values will be available in component.get()
      setTimeout(() => this.fire('columnWidthUpdated', {
        idx: __columnIndexBeingResized,
        width: newColumnWidth
      }), 0);
    }

    this.set(obj);
    
    // if still resizing and the user does not have the left mouse button depressed,
    // the mouseup event didn't fire for some reason, so turn off the resize mode
    
  },

  /**
   * Mouseup handler for column resizing
   */
  onColumnResizeEnd(event) {
    const { __resizing } = this.get();
    if (!__resizing) {
      return;
    }
    
    this.fire('columnWidthUpdated');
    this.set({
      __resizing: false,
      __columnIndexBeingResized: null
    });
  },

  /**
   * Sets updated scroll values when the scrollable area is scrolled
   */
  onScroll() {
    const obj = {};
    // get current saved scroll values
    const { __scrollTop, __scrollLeft } = this.get();

    // get new scroll values from the scroll area
    const { scrollTop: newScrollTop, scrollLeft: newScrollLeft } = this.refs.tableSpace;

    /* 
     * To avoid doing unnecessary re-calculation of computed variables, don't set the scroll
     * properties that haven't changed
     */
    if (__scrollTop !== newScrollTop) {
      obj.__scrollTop = newScrollTop;
    }

    if (__scrollLeft !== newScrollLeft) {
      obj.__scrollLeft = newScrollLeft;
    }

    obj.__scrolledAllTheWayToTheRight = Math.ceil(this.refs.tableSpace.scrollWidth - this.refs.tableSpace.scrollLeft) === this.refs.tableSpace.clientWidth;

    this.set(obj);
  }
};

function oncreate() {
  const { rows } = this.get();
  this.editHistory = new EditHistory(rows);
}
function onupdate({ changed, current, previous }) {
  // Record the change in onupdate to allow the DOM to change before doing the deep diff
  if (changed.rows && previous && previous.rows && previous.rows.length > 0) {
    if (!current.skipRecord) {
      this.editHistory.recordChange(current.rows);
    } else {
      this.set({ skipRecord: false });
    }
  }

  
}
function dragCopy(node, enabled) {
  let copy = null;
  let dragging = false;
  let offsetX = 0;

  function onWindowMouseMove(event) {
    if (!dragging) {
      return;
    }
    copy.style.left = (event.pageX - offsetX) + 'px';
  }

  function onWindowMouseUp(event) {
    if (!dragging || event.which !== 1) {
      return;
    }

    dragging = false;
    document.body.removeChild(copy);
    copy = null;
  }

  function onNodeMouseDown(event) {
    if (event.which !== 1) {
      return;
    }

    dragging = true;
    if (copy) {
      document.body.removeChild(copy);
    }

    copy = createCopy();
    offsetX = event.offsetX;
    copy.style.top = (node.getBoundingClientRect().top + getBodyScrollTop()) + 'px';
    copy.style.left = (event.pageX - offsetX) + 'px';
    document.body.appendChild(copy);
  }

  function createCopy() {
    const copy = document.createElement('div');
    copy.innerHTML = node.innerHTML;
    const { width, height, textAlign, fontWeight } = getComputedStyle(node);
    copy.style.width = width;
    copy.style.height = height;
    copy.style.maxHeight = height;
    copy.style.textAlign = textAlign;
    copy.style.fontWeight = fontWeight;
    copy.style.position = 'absolute';
    copy.style.opacity = '0.5';
    copy.style.pointerEvents = 'none';
    copy.style.overflow = 'hidden';
    copy.style.background = '#dddddd';
    copy.style['z-index'] = '99999';

    return copy;
  }

  function attachEvents() {
    window.addEventListener('mousemove', onWindowMouseMove);
    window.addEventListener('mouseup', onWindowMouseUp);
    node.addEventListener('mousedown', onNodeMouseDown);
  }

  function detachEvents() {
    window.removeEventListener('mousemove', onWindowMouseMove);
    window.removeEventListener('mouseup', onWindowMouseUp);
    node.removeEventListener('mousedown', onNodeMouseDown);
  }
  if (enabled) {
    attachEvents();
  }

  return {
    destroy() {
      detachEvents();
    },
    update(enabled) {
      if (enabled) {
        attachEvents();
      } else {
        detachEvents();
      }
    }
  }
}
function add_css() {
	var style = createElement("style");
	style.id = 'svelte-1f9vql4-style';
	style.textContent = ".row-action-line.svelte-1f9vql4{position:absolute;left:0;width:100%;height:4px;z-index:6;background:#aaa}.row-affix-marker.svelte-1f9vql4{display:none;position:absolute;left:0;height:4px;z-index:6;background:white;border-top:1px solid #999;border-bottom:1px solid #999;cursor:ns-resize}.column-affix-marker.svelte-1f9vql4{position:absolute;top:0;width:4px;z-index:5;background:white;border-left:1px solid #999;border-right:1px solid #999;cursor:ew-resize;transform:translateX(-50%)}.resizing.svelte-1f9vql4 .svelte-1f9vql4{user-select:none}.resizing.svelte-1f9vql4 .grid-inner.svelte-1f9vql4{overflow-y:hidden}.resizing.svelte-1f9vql4 .grid-space.svelte-1f9vql4{pointer-events:all}.grid-cell.svelte-1f9vql4>.svelte-1f9vql4{height:100%}.cell-default.svelte-1f9vql4{padding:0 5px;background:white;overflow:hidden;text-overflow:ellipsis}.data-grid-wrapper.svelte-1f9vql4{position:relative;width:100%;height:100%}.column-action-line.svelte-1f9vql4{position:absolute;top:0;bottom:17px;z-index:3;width:4px;background:#aaa;cursor:ew-resize}.grid-cell-size-capture.svelte-1f9vql4{position:absolute;top:0;bottom:0;z-index:5;background:transparent;cursor:ew-resize;pointer-events:all}.grid-inner.svelte-1f9vql4{overflow:auto}.grid-space.svelte-1f9vql4{position:absolute;top:0;left:0;background:transparent;pointer-events:none;z-index:3}.grid-headers.svelte-1f9vql4{position:absolute;overflow:hidden;max-width:100%;width:100%;top:0;left:0;border-bottom:2px solid black}.grid-headers.svelte-1f9vql4 .grid-cell.svelte-1f9vql4{text-align:center;font-weight:bold;cursor:pointer}.grid-headers.svelte-1f9vql4 .cell-default.svelte-1f9vql4:hover{background:#eee}.grid-header-row.svelte-1f9vql4{position:absolute;overflow:hidden;top:0}.grid-row.svelte-1f9vql4{position:absolute;overflow:hidden;white-space:nowrap;text-overflow:ellipsis}.grid-row.svelte-1f9vql4:not(:last-child){border-bottom:1px solid #ccc}.grid-cell.svelte-1f9vql4{position:absolute;top:0;text-overflow:ellipsis;overflow:hidden}.grid-cell.svelte-1f9vql4>.svelte-1f9vql4{width:100%}.grid-cell.svelte-1f9vql4:not(:last-child){border-right:1px solid #ccc}";
	append(document.head, style);
}

function get_each_context_1(ctx, list, i) {
	const child_ctx = Object.create(ctx);
	child_ctx.column = list[i];
	child_ctx.j = i;
	return child_ctx;
}

function get_each1_context(ctx, list, i) {
	const child_ctx = Object.create(ctx);
	child_ctx.row = list[i];
	child_ctx.i = i;
	return child_ctx;
}

function mousedown_handler_2(event) {
	const { component, ctx } = this._svelte;

	component.onColumnResizeStart(event, ctx.i);
}

function get_each_context(ctx, list, i) {
	const child_ctx = Object.create(ctx);
	child_ctx.column = list[i];
	child_ctx.i = i;
	return child_ctx;
}

function mousedown_handler_1(event) {
	const { component, ctx } = this._svelte;

	component.onColumnResizeStart(event, ctx.i);
}

function mousedown_handler(event) {
	const { component, ctx } = this._svelte;

	component.onColumnDragStart(event, ctx.i);
}

function get_each0_context(ctx, list, i) {
	const child_ctx = Object.create(ctx);
	child_ctx.column = list[i];
	child_ctx.i = i;
	return child_ctx;
}

function create_main_fragment(component, ctx) {
	var div4, text0, text1, div1, div0, each0_blocks_1 = [], each0_lookup = blankObject(), text2, div3, text3, div2, text4, div3_resize_listener, div4_class_value;

	function onwindowmouseup(event) {
		component.onMouseUp(event);	}
	window.addEventListener("mouseup", onwindowmouseup);

	function onwindowmousemove(event) {
		component.onMouseMove(event);	}
	window.addEventListener("mousemove", onwindowmousemove);

	function onwindowkeydown(event) {
		component.onWindowKeyDown(event);	}
	window.addEventListener("keydown", onwindowkeydown);

	var if_block0 = (ctx.__resizing || ctx.__columnDragging || ctx.__affixingColumn) && create_if_block_7(component, ctx);

	var if_block1 = (ctx.__affixingRow) && create_if_block_6(component, ctx);

	var each0_value = ctx.columns;

	const get_key = ctx => ctx.i;

	for (var i_1 = 0; i_1 < each0_value.length; i_1 += 1) {
		let child_ctx = get_each0_context(ctx, each0_value, i_1);
		let key = get_key(child_ctx);
		each0_blocks_1[i_1] = each0_lookup[key] = create_each_block_3(component, key, child_ctx);
	}

	var if_block2 = (ctx.allowColumnAffix) && create_if_block_3(component, ctx);

	var if_block3 = (ctx.allowResizeFromTableCells) && create_if_block_1(component, ctx);

	var each1_value = ctx.visibleRows;

	var each1_blocks = [];

	for (var i_1 = 0; i_1 < each1_value.length; i_1 += 1) {
		each1_blocks[i_1] = create_each_block(component, get_each1_context(ctx, each1_value, i_1));
	}

	function div3_resize_handler() {
		component.set({ __innerOffsetHeight: div3.offsetHeight });
	}

	function scroll_handler(event) {
		component.onScroll();
	}

	return {
		c() {
			div4 = createElement("div");
			if (if_block0) if_block0.c();
			text0 = createText("\n  ");
			if (if_block1) if_block1.c();
			text1 = createText("\n\n  ");
			div1 = createElement("div");
			div0 = createElement("div");

			for (i_1 = 0; i_1 < each0_blocks_1.length; i_1 += 1) each0_blocks_1[i_1].c();

			text2 = createText("\n\n  ");
			div3 = createElement("div");
			if (if_block2) if_block2.c();
			text3 = createText("\n    \n\n    \n    ");
			div2 = createElement("div");
			if (if_block3) if_block3.c();
			text4 = createText("\n    \n    \n    \n    \n    ");

			for (var i_1 = 0; i_1 < each1_blocks.length; i_1 += 1) {
				each1_blocks[i_1].c();
			}
			div0.className = "grid-header-row svelte-1f9vql4";
			setStyle(div0, "left", "-" + ctx.__scrollLeft + "px");
			setStyle(div0, "height", "" + ctx.rowHeight + "px");
			setStyle(div0, "width", "" + ctx.gridSpaceWidth + "px");
			setAttribute(div0, "role", "row");
			div1.className = "grid-headers svelte-1f9vql4";
			setStyle(div1, "height", "" + ctx.rowHeight + "px");
			setAttribute(div1, "rolw", "rowgroup");
			div2.className = "grid-space svelte-1f9vql4";
			setStyle(div2, "width", "" + ctx.gridSpaceWidth + "px");
			setStyle(div2, "height", "" + ctx.gridSpaceHeight + "px");
			component.root._aftercreate.push(div3_resize_handler);
			addListener(div3, "scroll", scroll_handler);
			div3.className = "grid-inner svelte-1f9vql4";
			setStyle(div3, "height", "100%");
			setAttribute(div3, "role", "rowgroup");
			div4.className = div4_class_value = "data-grid-wrapper " + (ctx.__resizing || ctx.__columnDragging ? 'resizing' : '') + " svelte-1f9vql4";
			setStyle(div4, "padding-top", "" + ctx.rowHeight + "px");
			setAttribute(div4, "role", "table");
		},

		m(target, anchor) {
			insert(target, div4, anchor);
			if (if_block0) if_block0.m(div4, null);
			append(div4, text0);
			if (if_block1) if_block1.m(div4, null);
			append(div4, text1);
			append(div4, div1);
			append(div1, div0);

			for (i_1 = 0; i_1 < each0_blocks_1.length; i_1 += 1) each0_blocks_1[i_1].m(div0, null);

			append(div4, text2);
			append(div4, div3);
			if (if_block2) if_block2.m(div3, null);
			append(div3, text3);
			append(div3, div2);
			if (if_block3) if_block3.m(div2, null);
			append(div3, text4);

			for (var i_1 = 0; i_1 < each1_blocks.length; i_1 += 1) {
				each1_blocks[i_1].m(div3, null);
			}

			div3_resize_listener = addResizeListener(div3, div3_resize_handler);
			component.refs.tableSpace = div3;
			component.refs.wrapper = div4;
		},

		p(changed, ctx) {
			if (ctx.__resizing || ctx.__columnDragging || ctx.__affixingColumn) {
				if (if_block0) {
					if_block0.p(changed, ctx);
				} else {
					if_block0 = create_if_block_7(component, ctx);
					if_block0.c();
					if_block0.m(div4, text0);
				}
			} else if (if_block0) {
				if_block0.d(1);
				if_block0 = null;
			}

			if (ctx.__affixingRow) {
				if (if_block1) {
					if_block1.p(changed, ctx);
				} else {
					if_block1 = create_if_block_6(component, ctx);
					if_block1.c();
					if_block1.m(div4, text1);
				}
			} else if (if_block1) {
				if_block1.d(1);
				if_block1 = null;
			}

			const each0_value = ctx.columns;
			each0_blocks_1 = updateKeyedEach(each0_blocks_1, component, changed, get_key, 1, ctx, each0_value, each0_lookup, div0, destroyBlock, create_each_block_3, "m", null, get_each0_context);

			if (changed.__scrollLeft) {
				setStyle(div0, "left", "-" + ctx.__scrollLeft + "px");
			}

			if (changed.rowHeight) {
				setStyle(div0, "height", "" + ctx.rowHeight + "px");
			}

			if (changed.gridSpaceWidth) {
				setStyle(div0, "width", "" + ctx.gridSpaceWidth + "px");
			}

			if (changed.rowHeight) {
				setStyle(div1, "height", "" + ctx.rowHeight + "px");
			}

			if (ctx.allowColumnAffix) {
				if (if_block2) {
					if_block2.p(changed, ctx);
				} else {
					if_block2 = create_if_block_3(component, ctx);
					if_block2.c();
					if_block2.m(div3, text3);
				}
			} else if (if_block2) {
				if_block2.d(1);
				if_block2 = null;
			}

			if (ctx.allowResizeFromTableCells) {
				if (if_block3) {
					if_block3.p(changed, ctx);
				} else {
					if_block3 = create_if_block_1(component, ctx);
					if_block3.c();
					if_block3.m(div2, null);
				}
			} else if (if_block3) {
				if_block3.d(1);
				if_block3 = null;
			}

			if (changed.gridSpaceWidth) {
				setStyle(div2, "width", "" + ctx.gridSpaceWidth + "px");
			}

			if (changed.gridSpaceHeight) {
				setStyle(div2, "height", "" + ctx.gridSpaceHeight + "px");
			}

			if (changed.visibleRows || changed.rowHeight || changed.gridSpaceWidth || changed.columns || changed.__affixedColumnIndices || changed.columnWidths || changed.__scrollLeft) {
				each1_value = ctx.visibleRows;

				for (var i_1 = 0; i_1 < each1_value.length; i_1 += 1) {
					const child_ctx = get_each1_context(ctx, each1_value, i_1);

					if (each1_blocks[i_1]) {
						each1_blocks[i_1].p(changed, child_ctx);
					} else {
						each1_blocks[i_1] = create_each_block(component, child_ctx);
						each1_blocks[i_1].c();
						each1_blocks[i_1].m(div3, null);
					}
				}

				for (; i_1 < each1_blocks.length; i_1 += 1) {
					each1_blocks[i_1].d(1);
				}
				each1_blocks.length = each1_value.length;
			}

			if ((changed.__resizing || changed.__columnDragging) && div4_class_value !== (div4_class_value = "data-grid-wrapper " + (ctx.__resizing || ctx.__columnDragging ? 'resizing' : '') + " svelte-1f9vql4")) {
				div4.className = div4_class_value;
			}

			if (changed.rowHeight) {
				setStyle(div4, "padding-top", "" + ctx.rowHeight + "px");
			}
		},

		d(detach) {
			window.removeEventListener("mouseup", onwindowmouseup);

			window.removeEventListener("mousemove", onwindowmousemove);

			window.removeEventListener("keydown", onwindowkeydown);

			if (detach) {
				detachNode(div4);
			}

			if (if_block0) if_block0.d();
			if (if_block1) if_block1.d();

			for (i_1 = 0; i_1 < each0_blocks_1.length; i_1 += 1) each0_blocks_1[i_1].d();

			if (if_block2) if_block2.d();
			if (if_block3) if_block3.d();

			destroyEach(each1_blocks, detach);

			div3_resize_listener.cancel();
			removeListener(div3, "scroll", scroll_handler);
			if (component.refs.tableSpace === div3) component.refs.tableSpace = null;
			if (component.refs.wrapper === div4) component.refs.wrapper = null;
		}
	};
}

// (3:2) {#if __resizing || __columnDragging || __affixingColumn}
function create_if_block_7(component, ctx) {
	var div;

	return {
		c() {
			div = createElement("div");
			div.className = "column-action-line svelte-1f9vql4";
			setStyle(div, "left", "" + (ctx.__columnActionLineLeft - 2) + "px");
		},

		m(target, anchor) {
			insert(target, div, anchor);
		},

		p(changed, ctx) {
			if (changed.__columnActionLineLeft) {
				setStyle(div, "left", "" + (ctx.__columnActionLineLeft - 2) + "px");
			}
		},

		d(detach) {
			if (detach) {
				detachNode(div);
			}
		}
	};
}

// (6:2) {#if __affixingRow}
function create_if_block_6(component, ctx) {
	var div;

	return {
		c() {
			div = createElement("div");
			div.className = "row-action-line svelte-1f9vql4";
			setStyle(div, "top", "" + (ctx.__rowActionLineTop - 2) + "px");
		},

		m(target, anchor) {
			insert(target, div, anchor);
		},

		p(changed, ctx) {
			if (changed.__rowActionLineTop) {
				setStyle(div, "top", "" + (ctx.__rowActionLineTop - 2) + "px");
			}
		},

		d(detach) {
			if (detach) {
				detachNode(div);
			}
		}
	};
}

// (19:10) {:else}
function create_else_block_1(component, ctx) {
	var div, text_value = ctx.column.display || '', text;

	return {
		c() {
			div = createElement("div");
			text = createText(text_value);
			div.className = "cell-default svelte-1f9vql4";
		},

		m(target, anchor) {
			insert(target, div, anchor);
			append(div, text);
		},

		p(changed, ctx) {
			if ((changed.columns) && text_value !== (text_value = ctx.column.display || '')) {
				setData(text, text_value);
			}
		},

		d(detach) {
			if (detach) {
				detachNode(div);
			}
		}
	};
}

// (17:10) {#if column.headerComponent}
function create_if_block_5(component, ctx) {
	var switch_instance_anchor;

	var switch_value = ctx.column.headerComponent;

	function switch_props(ctx) {
		var switch_instance_initial_data = { column: ctx.column };
		return {
			root: component.root,
			store: component.store,
			data: switch_instance_initial_data
		};
	}

	if (switch_value) {
		var switch_instance = new switch_value(switch_props(ctx));
	}

	return {
		c() {
			if (switch_instance) switch_instance._fragment.c();
			switch_instance_anchor = createComment();
		},

		m(target, anchor) {
			if (switch_instance) {
				switch_instance._mount(target, anchor);
			}

			insert(target, switch_instance_anchor, anchor);
		},

		p(changed, ctx) {
			var switch_instance_changes = {};
			if (changed.columns) switch_instance_changes.column = ctx.column;

			if (switch_value !== (switch_value = ctx.column.headerComponent)) {
				if (switch_instance) {
					switch_instance.destroy();
				}

				if (switch_value) {
					switch_instance = new switch_value(switch_props(ctx));
					switch_instance._fragment.c();
					switch_instance._mount(switch_instance_anchor.parentNode, switch_instance_anchor);
				} else {
					switch_instance = null;
				}
			}

			else if (switch_value) {
				switch_instance._set(switch_instance_changes);
			}
		},

		d(detach) {
			if (detach) {
				detachNode(switch_instance_anchor);
			}

			if (switch_instance) switch_instance.destroy(detach);
		}
	};
}

// (25:8) {#if allowResizeFromTableHeaders && !column.disallowResize}
function create_if_block_4(component, ctx) {
	var div;

	return {
		c() {
			div = createElement("div");
			div._svelte = { component, ctx };

			addListener(div, "mousedown", mousedown_handler_1);
			div.className = "grid-cell-size-capture svelte-1f9vql4";
			setStyle(div, "left", "" + (getCellLeft({i: ctx.i, columnWidths: ctx.columnWidths, __affixedColumnIndices: ctx.__affixedColumnIndices, __scrollLeft: ctx.__scrollLeft}) + ctx.columnWidths[ctx.i] - ctx.Math.floor(ctx.__columnHeaderResizeCaptureWidth / 2)) + "px");
			setStyle(div, "width", "" + ctx.__columnHeaderResizeCaptureWidth + "px");
		},

		m(target, anchor) {
			insert(target, div, anchor);
		},

		p(changed, _ctx) {
			ctx = _ctx;
			div._svelte.ctx = ctx;
			if (changed.columns || changed.columnWidths || changed.__affixedColumnIndices || changed.__scrollLeft || changed.Math || changed.__columnHeaderResizeCaptureWidth) {
				setStyle(div, "left", "" + (getCellLeft({i: ctx.i, columnWidths: ctx.columnWidths, __affixedColumnIndices: ctx.__affixedColumnIndices, __scrollLeft: ctx.__scrollLeft}) + ctx.columnWidths[ctx.i] - ctx.Math.floor(ctx.__columnHeaderResizeCaptureWidth / 2)) + "px");
			}

			if (changed.__columnHeaderResizeCaptureWidth) {
				setStyle(div, "width", "" + ctx.__columnHeaderResizeCaptureWidth + "px");
			}
		},

		d(detach) {
			if (detach) {
				detachNode(div);
			}

			removeListener(div, "mousedown", mousedown_handler_1);
		}
	};
}

// (15:6) {#each columns as column, i (i)}
function create_each_block_3(component, key_1, ctx) {
	var div, div_title_value, dragCopy_action, text, if_block1_anchor;

	function select_block_type(ctx) {
		if (ctx.column.headerComponent) return create_if_block_5;
		return create_else_block_1;
	}

	var current_block_type = select_block_type(ctx);
	var if_block0 = current_block_type(component, ctx);

	var if_block1 = (ctx.allowResizeFromTableHeaders && !ctx.column.disallowResize) && create_if_block_4(component, ctx);

	return {
		key: key_1,

		first: null,

		c() {
			div = createElement("div");
			if_block0.c();
			text = createText("\n        ");
			if (if_block1) if_block1.c();
			if_block1_anchor = createComment();
			div._svelte = { component, ctx };

			addListener(div, "mousedown", mousedown_handler);
			div.className = "grid-cell svelte-1f9vql4";
			setStyle(div, "z-index", getCellZIndex(ctx.__affixedColumnIndices, ctx.i));
			setStyle(div, "left", "" + getCellLeft({i: ctx.i, columnWidths: ctx.columnWidths, __affixedColumnIndices: ctx.__affixedColumnIndices, __scrollLeft: ctx.__scrollLeft}) + "px");
			setStyle(div, "width", "" + ctx.columnWidths[ctx.i] + "px");
			setStyle(div, "height", "" + ctx.rowHeight + "px");
			setStyle(div, "line-height", "" + ctx.rowHeight + "px");
			div.title = div_title_value = ctx.column.display || '';
			setAttribute(div, "role", "columnheader");
			this.first = div;
		},

		m(target, anchor) {
			insert(target, div, anchor);
			if_block0.m(div, null);
			dragCopy_action = dragCopy.call(component, div, ctx.allowColumnReordering) || {};
			insert(target, text, anchor);
			if (if_block1) if_block1.m(target, anchor);
			insert(target, if_block1_anchor, anchor);
		},

		p(changed, _ctx) {
			ctx = _ctx;
			if (current_block_type === (current_block_type = select_block_type(ctx)) && if_block0) {
				if_block0.p(changed, ctx);
			} else {
				if_block0.d(1);
				if_block0 = current_block_type(component, ctx);
				if_block0.c();
				if_block0.m(div, null);
			}

			div._svelte.ctx = ctx;
			if (changed.__affixedColumnIndices || changed.columns) {
				setStyle(div, "z-index", getCellZIndex(ctx.__affixedColumnIndices, ctx.i));
			}

			if (changed.columns || changed.columnWidths || changed.__affixedColumnIndices || changed.__scrollLeft) {
				setStyle(div, "left", "" + getCellLeft({i: ctx.i, columnWidths: ctx.columnWidths, __affixedColumnIndices: ctx.__affixedColumnIndices, __scrollLeft: ctx.__scrollLeft}) + "px");
			}

			if (changed.columnWidths || changed.columns) {
				setStyle(div, "width", "" + ctx.columnWidths[ctx.i] + "px");
			}

			if (changed.rowHeight) {
				setStyle(div, "height", "" + ctx.rowHeight + "px");
				setStyle(div, "line-height", "" + ctx.rowHeight + "px");
			}

			if ((changed.columns) && div_title_value !== (div_title_value = ctx.column.display || '')) {
				div.title = div_title_value;
			}

			if (typeof dragCopy_action.update === 'function' && changed.allowColumnReordering) {
				dragCopy_action.update.call(component, ctx.allowColumnReordering);
			}

			if (ctx.allowResizeFromTableHeaders && !ctx.column.disallowResize) {
				if (if_block1) {
					if_block1.p(changed, ctx);
				} else {
					if_block1 = create_if_block_4(component, ctx);
					if_block1.c();
					if_block1.m(if_block1_anchor.parentNode, if_block1_anchor);
				}
			} else if (if_block1) {
				if_block1.d(1);
				if_block1 = null;
			}
		},

		d(detach) {
			if (detach) {
				detachNode(div);
			}

			if_block0.d();
			removeListener(div, "mousedown", mousedown_handler);
			if (dragCopy_action && typeof dragCopy_action.destroy === 'function') dragCopy_action.destroy.call(component);
			if (detach) {
				detachNode(text);
			}

			if (if_block1) if_block1.d(detach);
			if (detach) {
				detachNode(if_block1_anchor);
			}
		}
	};
}

// (34:4) {#if allowColumnAffix}
function create_if_block_3(component, ctx) {
	var div;

	function mousedown_handler_2(event) {
		component.onColumnAffixStart(event);
	}

	return {
		c() {
			div = createElement("div");
			addListener(div, "mousedown", mousedown_handler_2);
			div.className = "column-affix-marker svelte-1f9vql4";
			setStyle(div, "left", "" + ctx.columnAffixLineLeft + "px");
			setStyle(div, "height", "" + ctx.gridSpaceHeight + "px");
		},

		m(target, anchor) {
			insert(target, div, anchor);
		},

		p(changed, ctx) {
			if (changed.columnAffixLineLeft) {
				setStyle(div, "left", "" + ctx.columnAffixLineLeft + "px");
			}

			if (changed.gridSpaceHeight) {
				setStyle(div, "height", "" + ctx.gridSpaceHeight + "px");
			}
		},

		d(detach) {
			if (detach) {
				detachNode(div);
			}

			removeListener(div, "mousedown", mousedown_handler_2);
		}
	};
}

// (41:6) {#if allowResizeFromTableCells}
function create_if_block_1(component, ctx) {
	var each_anchor;

	var each_value = ctx.columns;

	var each_blocks = [];

	for (var i_1 = 0; i_1 < each_value.length; i_1 += 1) {
		each_blocks[i_1] = create_each_block_2(component, get_each_context(ctx, each_value, i_1));
	}

	return {
		c() {
			for (var i_1 = 0; i_1 < each_blocks.length; i_1 += 1) {
				each_blocks[i_1].c();
			}

			each_anchor = createComment();
		},

		m(target, anchor) {
			for (var i_1 = 0; i_1 < each_blocks.length; i_1 += 1) {
				each_blocks[i_1].m(target, anchor);
			}

			insert(target, each_anchor, anchor);
		},

		p(changed, ctx) {
			if (changed.columns || changed.columnWidths || changed.__affixedColumnIndices || changed.__scrollLeft || changed.Math || changed.__columnHeaderResizeCaptureWidth) {
				each_value = ctx.columns;

				for (var i_1 = 0; i_1 < each_value.length; i_1 += 1) {
					const child_ctx = get_each_context(ctx, each_value, i_1);

					if (each_blocks[i_1]) {
						each_blocks[i_1].p(changed, child_ctx);
					} else {
						each_blocks[i_1] = create_each_block_2(component, child_ctx);
						each_blocks[i_1].c();
						each_blocks[i_1].m(each_anchor.parentNode, each_anchor);
					}
				}

				for (; i_1 < each_blocks.length; i_1 += 1) {
					each_blocks[i_1].d(1);
				}
				each_blocks.length = each_value.length;
			}
		},

		d(detach) {
			destroyEach(each_blocks, detach);

			if (detach) {
				detachNode(each_anchor);
			}
		}
	};
}

// (43:10) {#if !column.disallowResize}
function create_if_block_2(component, ctx) {
	var div;

	return {
		c() {
			div = createElement("div");
			div._svelte = { component, ctx };

			addListener(div, "mousedown", mousedown_handler_2);
			div.className = "grid-cell-size-capture svelte-1f9vql4";
			setStyle(div, "left", "" + (getCellLeft({i: ctx.i+1, columnWidths: ctx.columnWidths, __affixedColumnIndices: ctx.__affixedColumnIndices, __scrollLeft: ctx.__scrollLeft}) - ctx.Math.floor(ctx.__columnHeaderResizeCaptureWidth / 2)) + "px");
			setStyle(div, "width", "" + ctx.__columnHeaderResizeCaptureWidth + "px");
		},

		m(target, anchor) {
			insert(target, div, anchor);
		},

		p(changed, _ctx) {
			ctx = _ctx;
			div._svelte.ctx = ctx;
			if (changed.columnWidths || changed.__affixedColumnIndices || changed.__scrollLeft || changed.Math || changed.__columnHeaderResizeCaptureWidth) {
				setStyle(div, "left", "" + (getCellLeft({i: ctx.i+1, columnWidths: ctx.columnWidths, __affixedColumnIndices: ctx.__affixedColumnIndices, __scrollLeft: ctx.__scrollLeft}) - ctx.Math.floor(ctx.__columnHeaderResizeCaptureWidth / 2)) + "px");
			}

			if (changed.__columnHeaderResizeCaptureWidth) {
				setStyle(div, "width", "" + ctx.__columnHeaderResizeCaptureWidth + "px");
			}
		},

		d(detach) {
			if (detach) {
				detachNode(div);
			}

			removeListener(div, "mousedown", mousedown_handler_2);
		}
	};
}

// (42:8) {#each columns as column, i}
function create_each_block_2(component, ctx) {
	var if_block_anchor;

	var if_block = (!ctx.column.disallowResize) && create_if_block_2(component, ctx);

	return {
		c() {
			if (if_block) if_block.c();
			if_block_anchor = createComment();
		},

		m(target, anchor) {
			if (if_block) if_block.m(target, anchor);
			insert(target, if_block_anchor, anchor);
		},

		p(changed, ctx) {
			if (!ctx.column.disallowResize) {
				if (if_block) {
					if_block.p(changed, ctx);
				} else {
					if_block = create_if_block_2(component, ctx);
					if_block.c();
					if_block.m(if_block_anchor.parentNode, if_block_anchor);
				}
			} else if (if_block) {
				if_block.d(1);
				if_block = null;
			}
		},

		d(detach) {
			if (if_block) if_block.d(detach);
			if (detach) {
				detachNode(if_block_anchor);
			}
		}
	};
}

// (59:12) {:else}
function create_else_block(component, ctx) {
	var div, text_value = ctx.row.data[ctx.column.dataName] || '', text;

	return {
		c() {
			div = createElement("div");
			text = createText(text_value);
			div.className = "cell-default svelte-1f9vql4";
		},

		m(target, anchor) {
			insert(target, div, anchor);
			append(div, text);
		},

		p(changed, ctx) {
			if ((changed.visibleRows || changed.columns) && text_value !== (text_value = ctx.row.data[ctx.column.dataName] || '')) {
				setData(text, text_value);
			}
		},

		d(detach) {
			if (detach) {
				detachNode(div);
			}
		}
	};
}

// (57:12) {#if column.cellComponent}
function create_if_block(component, ctx) {
	var switch_instance_anchor;

	var switch_value = ctx.column.cellComponent;

	function switch_props(ctx) {
		var switch_instance_initial_data = {
		 	rowNumber: ctx.row.i,
		 	column: ctx.column,
		 	row: ctx.row
		 };
		return {
			root: component.root,
			store: component.store,
			data: switch_instance_initial_data
		};
	}

	if (switch_value) {
		var switch_instance = new switch_value(switch_props(ctx));
	}

	function switch_instance_valueupdate(event) {
		component.onCellUpdated(event);
	}

	if (switch_instance) switch_instance.on("valueupdate", switch_instance_valueupdate);

	return {
		c() {
			if (switch_instance) switch_instance._fragment.c();
			switch_instance_anchor = createComment();
		},

		m(target, anchor) {
			if (switch_instance) {
				switch_instance._mount(target, anchor);
			}

			insert(target, switch_instance_anchor, anchor);
		},

		p(changed, ctx) {
			var switch_instance_changes = {};
			if (changed.visibleRows) switch_instance_changes.rowNumber = ctx.row.i;
			if (changed.columns) switch_instance_changes.column = ctx.column;
			if (changed.visibleRows) switch_instance_changes.row = ctx.row;

			if (switch_value !== (switch_value = ctx.column.cellComponent)) {
				if (switch_instance) {
					switch_instance.destroy();
				}

				if (switch_value) {
					switch_instance = new switch_value(switch_props(ctx));
					switch_instance._fragment.c();
					switch_instance._mount(switch_instance_anchor.parentNode, switch_instance_anchor);

					switch_instance.on("valueupdate", switch_instance_valueupdate);
				} else {
					switch_instance = null;
				}
			}

			else if (switch_value) {
				switch_instance._set(switch_instance_changes);
			}
		},

		d(detach) {
			if (detach) {
				detachNode(switch_instance_anchor);
			}

			if (switch_instance) switch_instance.destroy(detach);
		}
	};
}

// (55:8) {#each columns as column, j}
function create_each_block_1(component, ctx) {
	var div;

	function select_block_type_1(ctx) {
		if (ctx.column.cellComponent) return create_if_block;
		return create_else_block;
	}

	var current_block_type = select_block_type_1(ctx);
	var if_block = current_block_type(component, ctx);

	return {
		c() {
			div = createElement("div");
			if_block.c();
			div.className = "grid-cell svelte-1f9vql4";
			setStyle(div, "z-index", getCellZIndex(ctx.__affixedColumnIndices, ctx.j));
			setStyle(div, "left", "" + getCellLeft({i: ctx.j, columnWidths: ctx.columnWidths, __affixedColumnIndices: ctx.__affixedColumnIndices, __scrollLeft: ctx.__scrollLeft}) + "px");
			setStyle(div, "height", "" + ctx.rowHeight + "px");
			setStyle(div, "line-height", "" + ctx.rowHeight + "px");
			setStyle(div, "width", "" + ctx.columnWidths[ctx.j] + "px");
			setAttribute(div, "role", "cell");
		},

		m(target, anchor) {
			insert(target, div, anchor);
			if_block.m(div, null);
		},

		p(changed, ctx) {
			if (current_block_type === (current_block_type = select_block_type_1(ctx)) && if_block) {
				if_block.p(changed, ctx);
			} else {
				if_block.d(1);
				if_block = current_block_type(component, ctx);
				if_block.c();
				if_block.m(div, null);
			}

			if (changed.__affixedColumnIndices) {
				setStyle(div, "z-index", getCellZIndex(ctx.__affixedColumnIndices, ctx.j));
			}

			if (changed.columnWidths || changed.__affixedColumnIndices || changed.__scrollLeft) {
				setStyle(div, "left", "" + getCellLeft({i: ctx.j, columnWidths: ctx.columnWidths, __affixedColumnIndices: ctx.__affixedColumnIndices, __scrollLeft: ctx.__scrollLeft}) + "px");
			}

			if (changed.rowHeight) {
				setStyle(div, "height", "" + ctx.rowHeight + "px");
				setStyle(div, "line-height", "" + ctx.rowHeight + "px");
			}

			if (changed.columnWidths) {
				setStyle(div, "width", "" + ctx.columnWidths[ctx.j] + "px");
			}
		},

		d(detach) {
			if (detach) {
				detachNode(div);
			}

			if_block.d();
		}
	};
}

// (53:4) {#each visibleRows as row, i}
function create_each_block(component, ctx) {
	var div, text, div_aria_rowindex_value;

	var each_value_1 = ctx.columns;

	var each_blocks = [];

	for (var i_2 = 0; i_2 < each_value_1.length; i_2 += 1) {
		each_blocks[i_2] = create_each_block_1(component, get_each_context_1(ctx, each_value_1, i_2));
	}

	return {
		c() {
			div = createElement("div");

			for (var i_2 = 0; i_2 < each_blocks.length; i_2 += 1) {
				each_blocks[i_2].c();
			}

			text = createText("\n      ");
			div.className = "grid-row svelte-1f9vql4";
			setStyle(div, "top", "" + getRowTop(ctx.row.i, ctx.rowHeight) + "px");
			setStyle(div, "height", "" + ctx.rowHeight + "px");
			setStyle(div, "width", "" + ctx.gridSpaceWidth + "px");
			setAttribute(div, "role", "row");
			setAttribute(div, "aria-rowindex", div_aria_rowindex_value = ctx.row.i);
		},

		m(target, anchor) {
			insert(target, div, anchor);

			for (var i_2 = 0; i_2 < each_blocks.length; i_2 += 1) {
				each_blocks[i_2].m(div, null);
			}

			append(div, text);
		},

		p(changed, ctx) {
			if (changed.__affixedColumnIndices || changed.columnWidths || changed.__scrollLeft || changed.rowHeight || changed.columns || changed.visibleRows) {
				each_value_1 = ctx.columns;

				for (var i_2 = 0; i_2 < each_value_1.length; i_2 += 1) {
					const child_ctx = get_each_context_1(ctx, each_value_1, i_2);

					if (each_blocks[i_2]) {
						each_blocks[i_2].p(changed, child_ctx);
					} else {
						each_blocks[i_2] = create_each_block_1(component, child_ctx);
						each_blocks[i_2].c();
						each_blocks[i_2].m(div, text);
					}
				}

				for (; i_2 < each_blocks.length; i_2 += 1) {
					each_blocks[i_2].d(1);
				}
				each_blocks.length = each_value_1.length;
			}

			if (changed.visibleRows || changed.rowHeight) {
				setStyle(div, "top", "" + getRowTop(ctx.row.i, ctx.rowHeight) + "px");
			}

			if (changed.rowHeight) {
				setStyle(div, "height", "" + ctx.rowHeight + "px");
			}

			if (changed.gridSpaceWidth) {
				setStyle(div, "width", "" + ctx.gridSpaceWidth + "px");
			}

			if ((changed.visibleRows) && div_aria_rowindex_value !== (div_aria_rowindex_value = ctx.row.i)) {
				setAttribute(div, "aria-rowindex", div_aria_rowindex_value);
			}
		},

		d(detach) {
			if (detach) {
				detachNode(div);
			}

			destroyEach(each_blocks, detach);
		}
	};
}

function Data_grid(options) {
	init(this, options);
	this.refs = {};
	this._state = assign(assign({ Math : Math }, data()), options.data);

	this._recompute({ columns: 1, __scrollLeft: 1, __affixedColumnIndices: 1, columnWidths: 1, rows: 1, __resizing: 1, __scrolledAllTheWayToTheRight: 1, rowHeight: 1, numRows: 1, __innerOffsetHeight: 1, __scrollTop: 1, numRowsInViewport: 1, __extraRows: 1 }, this._state);
	this._intro = true;
	this._handlers.update = [onupdate];

	if (!document.getElementById("svelte-1f9vql4-style")) add_css();

	this._fragment = create_main_fragment(this, this._state);

	this.root._oncreate.push(() => {
		oncreate.call(this);
		this.fire("update", { changed: assignTrue({}, this._state), current: this._state });
	});

	if (options.target) {
		this._fragment.c();
		this._mount(options.target, options.anchor);

		flush(this);
	}
}

assign(Data_grid.prototype, proto);
assign(Data_grid.prototype, methods);

Data_grid.prototype._recompute = function _recompute(changed, state) {
	if (changed.columns) {
		if (this._differs(state.columnWidths, (state.columnWidths = columnWidths_1(state)))) changed.columnWidths = true;
	}

	if (changed.__scrollLeft || changed.__affixedColumnIndices || changed.columnWidths) {
		if (this._differs(state.columnAffixLineLeft, (state.columnAffixLineLeft = columnAffixLineLeft(state)))) changed.columnAffixLineLeft = true;
	}

	if (changed.rows) {
		if (this._differs(state.numRows, (state.numRows = numRows(state)))) changed.numRows = true;
	}

	if (changed.columnWidths || changed.__resizing || changed.__scrolledAllTheWayToTheRight) {
		if (this._differs(state.gridSpaceWidth, (state.gridSpaceWidth = gridSpaceWidth(state)))) changed.gridSpaceWidth = true;
	}

	if (changed.rowHeight || changed.numRows) {
		if (this._differs(state.gridSpaceHeight, (state.gridSpaceHeight = gridSpaceHeight(state)))) changed.gridSpaceHeight = true;
	}

	if (changed.__innerOffsetHeight || changed.rowHeight) {
		if (this._differs(state.numRowsInViewport, (state.numRowsInViewport = numRowsInViewport(state)))) changed.numRowsInViewport = true;
	}

	if (changed.rowHeight || changed.rows || changed.__scrollTop || changed.numRowsInViewport || changed.__extraRows) {
		if (this._differs(state.visibleRows, (state.visibleRows = visibleRows(state)))) changed.visibleRows = true;
	}
};

export default Data_grid;
